#
# hello.txrx.45.py

# detect rotation of disk based on
# step response transmit-receive hello-world
#    9600 baud FTDI interface
#
# Takuma Oami modify to the purpose
# fablab Vestmannaeyjar 4/8/14
#
# Neil Gershenfeld wrote original code
# CBA MIT 11/6/11
#
# (c) Massachusetts Institute of Technology 2011
# Permission granted for experimental and personal use;
# license for commercial sale available from MIT
#

from Tkinter import *
from math import *
import serial

WINDOW = 600 # window size
RADIUS = 0.4*WINDOW # length of line
eps = 0.75 # filter fraction
filt1 = 0.0 # filtered value(PB3)
filt2 = 0.0 # filtered value(PB4)

max1 = 5900 #maximum of PB3
min1 = 2700 #minimum of PB3
max2 = 4400 #maximum of PB4
min2 = 1000 #minimum of PB4

def idle(parent,canvas):
   global filt1, filt2, eps
   #
   # idle routine
   #
   byte2 = 0
   byte3 = 0
   byte4 = 0
   ser.flush()
   #
   # find framing 
   #
   while 1:
      byte1 = byte2
      byte2 = byte3
      byte3 = byte4
      byte4 = ord(ser.read())
      if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 == 4)):
         break
   #
   # read and plot
   #
   up_low1 = ord(ser.read())
   up_high1 = ord(ser.read())
   down_low1 = ord(ser.read())
   down_high1 = ord(ser.read())

   up_low2 = ord(ser.read())
   up_high2 = ord(ser.read())
   down_low2 = ord(ser.read())
   down_high2 = ord(ser.read())

   up_value1 = 256*up_high1 + up_low1
   down_value1 = 256*down_high1 + down_low1
   value1 = (up_value1 - down_value1)
   filt1 = (1-eps)*filt1 + eps*value1
   cover1 = 0.0
   if (filt1 - min1) >=0 :
       cover1 = (filt1 - min1) / (max1 - min1)

   up_value2 = 256*up_high2 + up_low2
   down_value2 = 256*down_high2 + down_low2
   value2 = (up_value2 - down_value2)
   filt2 = (1-eps)*filt2 + eps*value2
   cover2 = 0.0
   if (filt2 - min2) >= 0:
       cover2 = (filt2 - min2) / (max2 - min2)

   total_angle = 0

   if cover1 > 0 and cover2 <= 0:
       total_angle = 120*cover1
   elif cover1 > 0 and cover2 > 0:
       total_angle = (( 120*cover2+120 )+( 120*(1-cover1)+120 ))/2
   elif cover2 >0 and cover1 <= 0:
       total_angle = 120*(1-cover2) + 240

   x= RADIUS * cos(radians(total_angle) + pi/2) + 0.5 * WINDOW
   y= -RADIUS * sin(radians(total_angle) + pi/2) + 0.5 * WINDOW
   canvas.coords('hand', .5*WINDOW,.5*WINDOW,x,y)
   canvas.itemconfigure("angle",text="%.1f"%total_angle)
   canvas.update()
   parent.after_idle(idle,parent,canvas)

#
#  check command line arguments
#
if (len(sys.argv) != 2):
   print "command line: hello.rotate.45.py serial_port"
   sys.exit()
port = sys.argv[1]
#
# open serial port
#
ser = serial.Serial(port,9600)
ser.setDTR()
#
# set up GUI
#
root = Tk()
root.title('hello.rotate.45.py (q to exit)')
root.bind('q','exit')

canvas = Canvas(root, width=WINDOW, height=WINDOW, background='white')

canvas.create_text(.05*WINDOW,.05*WINDOW,text="0",font=("Helvetica", 24),tags="angle",fill="#000000")
canvas.create_line(.5*WINDOW,.5*WINDOW,.5*WINDOW,.5*WINDOW-RADIUS,tags="zero",width="2",fill="#0000b0")
canvas.create_line(.5*WINDOW,.5*WINDOW,.5*WINDOW,.5*WINDOW-RADIUS,tags="hand",width="2",fill="red")

canvas.pack()
#
# start idle loop
#
root.after(100,idle,root,canvas)
root.mainloop()
